﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CalcHelper
{
    /*
     * As it turns out decided not to use this sort override
     * at all. Sorting the new box format here amounts to 
     * sorting a bunch of words in a list with no definite
     * connection to a page on the document. The whole idea
     * of sorting was to sort in ascending page order which
     * worked fine when we had the old box format where the
     * page number of a box was built into the box number.
     * So box12_15 would be on page 12. But we do not know
     * what page box345 is on.
     * 
     * So we are still sorting where we were calling this 
     * from but are now using a dictionary of box keys and
     * their associated page number values, the dictionary is
     * traversed via a foreach and is ordered by key.Value.
     */
    public partial class Form1 : Form
    {
        /*
         * Coding for old box format was graciously provided by Joe Argiro.
         *
         * We use this by passing the reference of this class as an object
         * parameter to Sort() like this example using the new box format
         * as flagged by the BoxCompare argument newBoxFormat:
         *  
         * List<string> boxList = new List<string> { "box51", "box2", "box110", "box15" };
         *  
         * boxList.Sort(new BoxCompare(newBoxFormat));
         *  
         * I am passing the new box format to BoxCompare as I do not want
         * to have to figure out the box format every time we sort and for
         * every box being sorted in a list. We know the box format style
         * being used when CalcHelper is first launched and it does not change.
        */
        public class BoxCompare : IComparer<string>
        {
            private bool _newBoxFormat;

            public BoxCompare(bool newBoxFormat)
            {
                _newBoxFormat = newBoxFormat;
            }

            public int Compare(string p1, string p2)
            {
                int box1 = 0;
                int box2 = 0;
                int result = 0;

                if (_newBoxFormat == true)
                {
                    // New box format like box12
                    int.TryParse(p1.Substring(3), out box1);
                    int.TryParse(p2.Substring(3), out box2);

                    if (box1 == box2)
                        result = 0;
                    else if (box1 < box2)
                        result = -1;
                    else
                        result = 1;
                }
                else
                {
                    // We need pages for this old format.
                    int page1 = 0;
                    int page2 = 0;

                    // Old box format like box1_2
                    int.TryParse(p1.Substring(3, p1.Substring(3).IndexOf('_')), out page1);
                    int.TryParse(p1.Substring(p1.IndexOf('_') + 1), out box1);

                    int.TryParse(p2.Substring(3, p2.Substring(3).IndexOf('_')), out page2);
                    int.TryParse(p2.Substring(p2.IndexOf('_') + 1), out box2);

                    if (page1 == page2)
                    {
                        if (box1 == box2)
                            result = 0;
                        else if (box1 < box2)
                            result = -1;
                        else
                            result = 1;
                    }
                    else if (page1 < page2)
                    {
                        result = -1;
                    }
                    else
                        result = 1;
                }

                return result;
            }
        }
    }
}